"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.golangClient = exports.GolangClient = exports.GolangModelsBase = exports.GolangBaseDataModel = exports.GolangResponseDataModels = exports.GolangResponseModels = exports.GolangInputModels = void 0;
const codegen_1 = require("@undergraph-dev/sdk/internal/codegen");
const execa_1 = __importDefault(require("execa"));
const lodash_1 = require("lodash");
const handlebars_1 = __importDefault(require("handlebars"));
const client_template_1 = require("./client-template");
const protobuf_1 = require("@undergraph-dev/protobuf");
const logger_1 = __importDefault(require("@undergraph-dev/sdk/internal/logger"));
const logger = logger_1.default.child({ plugin: 'golang-client' });
const golangHeader = (packageName) => `// Code generated by wunderctl. DO NOT EDIT.\npackage ${packageName}\n\n`;
const defaultTemplateConfig = {
    packageName: 'client',
};
/**
 * Formats a given string with each line prepended by its line number
 * (starting at 1).
 *
 * @remarks
 * Lines are split by \r?\n, then joined by \n
 *
 * @param code - The code to split and prepend line numbers to
 * @returns A string with the line annotated code
 */
function linefy(code) {
    return code
        .split(/\r?\n/)
        .map((line, idx) => `${idx + 1}: ${line}`)
        .join('\n');
}
const gofmt = (code) => {
    try {
        // check if gofmt is installed
        const formatter = execa_1.default.sync('gofmt', {
            input: code,
            encoding: 'utf8',
        });
        if (formatter.exitCode === 0) {
            return formatter.stdout;
        }
    }
    catch (e) {
        // If the error is due to gofmt not being installed, we ignore it
        // on purpose. Otherwise we throw an error with both the code and
        // the error message returned by gofmt.
        if (e instanceof Error && e.message.indexOf('ENOENT') >= 0) {
            logger.warn('gofmt is not installed. If you want to prettify the generated code, please install gofmt');
        }
        else {
            throw new Error(`failed to format:\n${linefy(code)}\n\n${e}`);
        }
    }
    return code;
};
class GolangInputModels {
    constructor(config = Object.assign({}, defaultTemplateConfig)) {
        this.config = config;
    }
    async generate(generationConfig) {
        const content = generationConfig.config.application.Operations.filter(codegen_1.hasInput)
            .map((op) => JSONSchemaToGolangStruct(op.VariablesSchema, op.Name + 'Input', false))
            .join('\n\n');
        return Promise.resolve([
            {
                path: 'models.go',
                content: gofmt(content),
                header: golangHeader(this.config.packageName),
            },
        ]);
    }
    dependencies() {
        return [new GolangModelsBase()];
    }
}
exports.GolangInputModels = GolangInputModels;
class GolangResponseModels {
    constructor(config = Object.assign({}, defaultTemplateConfig)) {
        this.config = config;
    }
    generate(generationConfig) {
        const config = generationConfig.config;
        const content = config.application.Operations.map((op) => {
            const dataName = '#/definitions/' + op.Name + 'ResponseData';
            const responseSchema = {
                type: 'object',
                properties: {
                    data: {
                        $ref: dataName,
                    },
                },
            };
            return JSONSchemaToGolangStruct(responseSchema, op.Name + 'Response', true);
        }).join('\n\n');
        return Promise.resolve([
            {
                path: 'models.go',
                content: gofmt(content),
                header: golangHeader(this.config.packageName),
            },
        ]);
    }
    dependencies() {
        return [new GolangModelsBase(), new GolangResponseDataModels(this.config), new GolangBaseDataModel(this.config)];
    }
}
exports.GolangResponseModels = GolangResponseModels;
const responseDataSchema = (op) => {
    var _a, _b;
    switch (op.ExecutionEngine) {
        case protobuf_1.OperationExecutionEngine.ENGINE_NODEJS:
            return op.ResponseSchema;
        case protobuf_1.OperationExecutionEngine.ENGINE_GRAPHQL:
            return (_b = (_a = op.ResponseSchema) === null || _a === void 0 ? void 0 : _a.properties) === null || _b === void 0 ? void 0 : _b['data'];
    }
    throw new Error(`unhandled operation engine ${op.ExecutionEngine}`);
};
class GolangResponseDataModels {
    constructor(config = Object.assign({}, defaultTemplateConfig)) {
        this.config = config;
    }
    generate(generationConfig) {
        const content = generationConfig.config.application.Operations.filter((op) => responseDataSchema(op) !== undefined)
            .map((op) => JSONSchemaToGolangStruct(responseDataSchema(op), op.Name + 'ResponseData', false))
            .join('\n\n');
        return Promise.resolve([
            {
                path: 'models.go',
                content: gofmt(content),
                header: golangHeader(this.config.packageName),
            },
        ]);
    }
}
exports.GolangResponseDataModels = GolangResponseDataModels;
class GolangBaseDataModel {
    constructor(config = Object.assign({}, defaultTemplateConfig)) {
        this.config = config;
    }
    generate(generationConfig) {
        const definitions = new Map();
        generationConfig.config.application.Operations.forEach((op) => {
            if (!op.VariablesSchema.definitions) {
                return;
            }
            Object.keys(op.VariablesSchema.definitions).forEach((definitionName) => {
                if (definitions.has(definitionName)) {
                    return;
                }
                const definition = op.VariablesSchema.definitions[definitionName];
                if (typeof definition !== 'object') {
                    return;
                }
                definitions.set(definitionName, definition);
            });
        });
        const content = Array.from(definitions.entries())
            .map(([definitionName, definition]) => JSONSchemaToGolangStruct(definition, definitionName, false))
            .join('\n\n');
        return Promise.resolve([
            {
                path: 'models.go',
                content: gofmt(content),
                header: golangHeader(this.config.packageName),
            },
        ]);
    }
}
exports.GolangBaseDataModel = GolangBaseDataModel;
const golangModelsBase = `
type GraphQLError struct {
	Message string
	Path    []interface{}
}`;
class GolangModelsBase {
    async generate(config) {
        return Promise.resolve([
            {
                path: 'models.go',
                content: gofmt(golangModelsBase),
            },
        ]);
    }
    dependencies() {
        return [];
    }
}
exports.GolangModelsBase = GolangModelsBase;
class GolangClient {
    constructor(config = Object.assign({}, defaultTemplateConfig)) {
        this.config = config;
    }
    async generate(generationConfig) {
        const config = generationConfig.config;
        const tmpl = handlebars_1.default.compile(client_template_1.clientTemplate);
        const content = tmpl({
            queries: config.application.Operations.filter((op) => op.OperationType === protobuf_1.OperationType.QUERY).map((op) => ({
                name: op.Name,
                hasInput: (0, codegen_1.hasInput)(op),
            })),
            mutations: config.application.Operations.filter((op) => op.OperationType === protobuf_1.OperationType.MUTATION).map((op) => ({ name: op.Name, hasInput: (0, codegen_1.hasInput)(op) })),
            subscriptions: config.application.Operations.filter((op) => op.OperationType === protobuf_1.OperationType.SUBSCRIPTION).map((op) => ({ name: op.Name, hasInput: (0, codegen_1.hasInput)(op) })),
            baseURL: config.deployment.environment.baseUrl,
        });
        return Promise.resolve([
            {
                path: 'client.go',
                content: gofmt(content),
                header: golangHeader(this.config.packageName),
            },
        ]);
    }
}
exports.GolangClient = GolangClient;
const JSONSchemaToGolangStruct = (schema, structName, withErrors) => {
    let out = '';
    const capitalizeFirstChar = (name) => (0, lodash_1.capitalize)(name.substring(0, 1)) + name.substring(1);
    const addJsonTag = (fieldName, isArray) => {
        out += ` \`json:"${fieldName},omitempty"\`\n`;
    };
    (0, codegen_1.visitJSONSchema)(schema, {
        root: {
            enter: () => {
                out += `type ${capitalizeFirstChar(structName)} struct {\n`;
            },
            leave: () => {
                if (withErrors) {
                    out += `\terrors []GraphQLError \`json:"errors"\``;
                }
                out += '\n}\n';
            },
        },
        number: (name, isRequired, isArray) => {
            out += `\t${capitalizeFirstChar(name)} ${isArray ? '[]' : ''}${isRequired ? '' : '*'}float64`;
            addJsonTag(name, isArray);
        },
        array: {
            enter: (name, isRequired, isArray) => {
                out += `\t${capitalizeFirstChar(name)}`;
            },
        },
        string: (name, isRequired, isArray, enumValues) => {
            out += `\t${capitalizeFirstChar(name)} ${isArray ? '[]' : ''}${isRequired ? '' : '*'}string`;
            addJsonTag(name, isArray);
        },
        object: {
            enter: (name, isRequired, isArray) => {
                out += `\t${capitalizeFirstChar(name)} ${isArray ? '[]' : ''}${isRequired ? '' : '*'}struct {\n`;
            },
            leave: (name, isRequired, isArray) => {
                out += `\t}`;
                addJsonTag(name, isArray);
            },
        },
        boolean: (name, isRequired, isArray) => {
            out += `\t${capitalizeFirstChar(name)} ${isArray ? '[]' : ''}${isRequired ? '' : '*'}bool`;
            addJsonTag(name, isArray);
        },
        any: (name, isRequired, isArray) => {
            out += `\t${capitalizeFirstChar(name)} ${isArray ? '[]' : ''}${isRequired ? '' : '*'}interface{}`;
            addJsonTag(name, isArray);
        },
        customType: (name, typeName, isRequired, isArray) => {
            if (typeName.indexOf('{') >= 0) {
                // Anonymous type not representable by Go
                // TODO: Do beter with these at some point, use an approach like the Rust generator
                typeName = `interface{}`;
            }
            else {
                typeName = capitalizeFirstChar(typeName);
            }
            out += `\t${capitalizeFirstChar(name)} ${isArray ? '[]' : ''} ${isRequired ? '' : '*'}${typeName}`;
            addJsonTag(name, isArray);
        },
    });
    return out;
};
/*
    -------------------------------------
    Dependencies
    -------------------------------------
    GolangModelsBase => BaseTypeScriptDataModel + TypeScriptEnumModels ===> models.go
    GolangInputModels => GolangModelsBase ===> models.go
    GolangResponseModels => GolangModelsBase + GolangResponseDataModels + GolangBaseDataModels ===> models.go
    GolangClient => None ===> client.go
*/
exports.golangClient = {
    all: (config = defaultTemplateConfig) => [
        new GolangInputModels(config),
        new GolangResponseModels(config),
        new GolangClient(config),
    ],
};
//# sourceMappingURL=index.js.map